//Please refer to http://dansguardian.org/?page=copyright2
//for the license for this code.
//Written by Daniel Barron (daniel@jadeb.com).
//For support go to http://groups.yahoo.com/group/dansguardian

//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "autoconf/platform.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <iostream>
#include <arpa/inet.h>
#include <cstdio>
#include <string>
#include <unistd.h>
#include <cerrno>
#include "testguardian.hpp"
#include <syslog.h>
#include <pwd.h>
#include <fstream>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/resource.h>
#ifdef __BSD
	#include <sys/wait.h>
#else
	#include <wait.h>
#endif


OptionContainer o;
bool isDaemonised;

int main(int argc, char* argv[]) {
    std::string configfile = __CONFFILE;
    isDaemonised = false;
    bool filter = false;
    
    for (int i = 1; i < argc; i++) {
        if (*argv[i] == '-') {
            switch (argv[i][1]) {
                case 'P':
                    return 0;
                case 'v':
                    std::cout << "DansGuardian 2.5.3-4" << std::endl;
                    return 0;
                case 'c':
                    configfile = argv[i+1];
                    i++;
                    break;
                case 'f':
                    filter = true;
                    break;
                case 'h':
                    std::cout << "Usage: " << argv[0] << " [{-v|-P|-h|-c ConfigFileName|-f}]" << std::endl;
                    std::cout << "  -v gives the version number." << std::endl;
                    std::cout << "  -P displays the name and version of any 3rd party extensions compiled in." << std::endl;
                    std::cout << "  -h gives this message." << std::endl;
                    std::cout << "  -c allows you to specify a different configuration file location." << std::endl;
                    std::cout << "  -f Filter mode, only sends PASSED input to stdout." << std::endl;
                    return 0;
                default:
                    std::cout << "Usage: " << argv[0] << " [{-v|-P|-h|-c ConfigFileName|-f}]" << std::endl;
                    std::cout << "  -v gives the version number." << std::endl;
                    std::cout << "  -P displays the name and version of any 3rd party extensions compiled in." << std::endl;
                    std::cout << "  -h gives this message." << std::endl;
                    std::cout << "  -c allows you to specify a different configuration file location." << std::endl;
                    std::cout << "  -f Filter mode, only sends PASSED input to stdout." << std::endl;
                    return 0;
            }
        }
        else {
            std::cout << "Usage: " << argv[0] << " [{-v|-P|-h|-c ConfigFileName|-f}]" << std::endl;
            std::cout << "  -v gives the version number." << std::endl;
            std::cout << "  -P displays the name and version of any 3rd party extensions compiled in." << std::endl;
            std::cout << "  -h gives this message." << std::endl;
            std::cout << "  -c allows you to specify a different configuration file location." << std::endl;
            std::cout << "  -f Filter mode, only sends PASSED input to stdout." << std::endl;
            return 1;
        }
    }

    o.loglocation = LOGLOCATION;  // this is set at make time
    int testopts = open(configfile.c_str(), 0, O_RDONLY);
    if (testopts < 0) {
        syslog(LOG_ERR, "%s %s.","Error accessing ", configfile.c_str());
	cerr << "Error accessing " << configfile << std::endl;
	return 1;  // could not open conf file for reading, exit with error
    }
    close (testopts);

    if (!o.read(configfile.c_str())) {
        syslog(LOG_ERR, "%s","Error parsing the dansguardian.conf file or other DansGuardian configuration files");
	cerr << "Error parsing the dansguardian.conf file or other DansGuardian configuration files" << std::endl;
	return 1;  // OptionContainer class had an error reading the conf or
	           // other files so exit with error
    }

    std::deque<std::string> urlfile;

    try {
        std::string linebuffer;
        String temp;  // for tempory conversion and storage
        int j;  // counter
        while (!std::cin.eof()) {
            getline(std::cin, linebuffer);
            if (!std::cin.eof() && linebuffer.length() != 0) {
                if (linebuffer[0] != '#') {  // i.e. not commented out
                    temp = (char*)linebuffer.c_str();
                    if (temp.contains("#")) {
                        temp = temp.before("#");
                    }
                    temp.removeWhiteSpace();  // get rid of spaces at end of line
                    std::string result;
                    
// <MAGIC>
// Skipping user/ip exceptions
// Skipping all phrase checking (though that's a good thought... shouldn't be too hard, but not my job)
// Skipping file-type checking because it's faster with an awk script
    if (o.inexceptions(temp)) result = "SITEEXCEPTION";
    else if (o.inurlexceptions(temp)) result = "URLEXCEPTION";
    else if (o.inBannedRegExpURLList(temp) >= 0) result = "BANNEDREGEX";
    else if (o.inBannedURLList(temp) >= 0) result = "BANNEDURL";
    else if (o.inBannedSiteList(temp) >= 0) result = "BANNEDSITE";
    else result = "PASSED";
// </MAGIC>
                    
                    if (!filter)
                        std::cout << temp << "\t" << result << std::endl;
                    else if (filter && result == "PASSED")
                        std::cout << temp << std::endl;
                }
            }
        }
    } catch (exception& e) {
        std::cerr << e.what() << std::endl;                    
	    return false;
    }
    
    return false;
}
